/*
Copyright (c) 2001-2005 MakeHuman team www.makehuman.org
Author: Paolo Colombo

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#include <GL/gl.h>
#endif

#include "glut_interface.h"

#include "make_human.h"
#include "io.h"
#include "ui.h"



#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>


#pragma comment( linker, "/entry:\"mainCRTStartup\"" )  // set the entry point to be main()
//#pragma comment (linker, "/SUBSYSTEM:WINDOWS")
#pragma comment (linker, "/SUBSYSTEM:CONSOLE")

//#pragma comment (lib, "opengl32.lib")
//#pragma comment (lib, "glut32.lib")
//#pragma comment (lib, "glu32.lib")


// Window's position (top-left corner) and size
int window_X_position;
int window_Y_position;
int window_width = 800;
int window_height = 600;



/* Forward declarations */


/* Variables */
GLfloat rotx = 0, roty = 0, trax = 0.0, tray = -.2, traz = -10.5, focal = 35.0;
int ox = 0, oy = 0;
static int fullscreen = 0;
int grid = 0;
int axis = 1;
int wi,hi;
static int mot = 0;
#define PAN	1
#define ROT	2
#define ZOM 3
int anim = 0;




GLfloat left_light_position[] = {0.0, 20.0, -200.0, 0.0};
GLfloat right_light_position[] = {1500.0, .0, .0, 1.0};
GLfloat	back_light_position[] = {.0, 0.0, 1500.0, 1.0};
GLfloat white_light[] = {.4, .4, .4, 0.0};

GLfloat  light_ambient[4]      = {0.3, 0.3, 0.3, 1.0};


void reshape (int w, int h)
{

  glMatrixMode(GL_PROJECTION);
  //glLoadIdentity();
  glViewport(0, 0, w, h);
  glLoadIdentity();
  gluPerspective(35.0, (GLfloat) w/(GLfloat) h, 0.1, 100.0);
  //glLoadIdentity();
  glMatrixMode(GL_MODELVIEW);
  wi = w;
  hi = h;

}


void zoom(int x, int y){
  traz +=  (x-ox)/200.;
  traz -= (y-oy)/200.;
  //printf("traz %f\n", traz);
  ox = x; oy = y;
  glutPostRedisplay();
}


void pan(int x, int y) {
  trax +=  (x-ox)/200.;
  tray -= (y-oy)/200.;
  //printf("trax %f - ", trax);
  //printf("tray %f\n", tray);
  ox = x; oy = y;
  glutPostRedisplay();
}

void rotate(int x, int y) {
  rotx += y-oy;
  //printf("rotx %f - ", rotx);
  if (rotx > 360.) rotx -= 360.;
  else if (rotx < -360.) rotx += 360.;
  roty += x-ox;
  //printf("roty %f\n", roty);
  if (roty > 360.) roty -= 360.;
  else if (roty < -360.) roty += 360.;
  ox = x; oy = y;
  glutPostRedisplay();
}


void display(void)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glPushMatrix();
  glLoadIdentity();

  glTranslatef(trax, tray, traz);

  glRotatef(rotx, 1., 0., 0.);
  glRotatef(roty, 0., 1., 0.);

  if(grid)Draw_Grid();
  if(axis)Draw_Axis();

  glRotatef(-90, 1., 0., 0.);
  glScalef(.3,.3,.3);

  glEnable (GL_CULL_FACE);  // Do not calculate inside of mesh

  Draw_model();

  glDisable (GL_CULL_FACE);  // Do not calculate inside of mesh

  glPopMatrix();

  glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE,  white_light);
  glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
  glLightfv(GL_LIGHT0, GL_POSITION, left_light_position);

  glEnable(GL_LIGHT0);

  glEnable(GL_LIGHTING);
  glEnable(GL_COLOR_MATERIAL);

  glColor3f(0.0f,1.0f,1.0f);
  setOrthographicProjection(wi,hi);

  Draw_Decoration2(wi,hi);
  Draw_UI(wi,hi);

  resetPerspectiveProjection();

  //glFinish();
  glutSwapBuffers();
}


void motion(int x, int y) {
  if (mot == PAN) pan(x, y);
  else if (mot == ROT) rotate(x,y);
  else if (mot == ZOM) zoom(x,y);
  sliderUpdate(ox, x);
  ox = x;
}


void mouse(int button, int state, int x, int y) {
  if(state == GLUT_DOWN) {
    switch(button) {

    case GLUT_MIDDLE_BUTTON:
      mot = ROT;
      motion(ox = x, oy = y);
      break;
    case GLUT_LEFT_BUTTON:

      if(glutGetModifiers() & GLUT_ACTIVE_SHIFT){
        mot = PAN;
        motion(ox = x, oy = y);
      }
      else if(glutGetModifiers() & GLUT_ACTIVE_CTRL){
        mot = ROT;
        motion(ox = x, oy = y);
      }
      else if(glutGetModifiers() & GLUT_ACTIVE_ALT){
        mot = ZOM;
        motion(ox = x, oy = y);
      }
      else{
        OnClick(x,y);
        OnSlider(x,y);
        ox = x;
        oy = y;
      }
      break; /* don't forget to break after case ;-) */

    case GLUT_RIGHT_BUTTON:
      /* inserted zoom function hack for Linux. Most X windows managers grap ALT-Mouse key.
          We should take some better default keys or made them configurable
      */
      mot = ZOM;
      motion(ox = x, oy = y);
      break;

    }
  } else if (state == GLUT_UP) {
    mot = 0;
    OnRelease();
  }
}


void toggle_fullscreen()
{
	// Toggle fullscreen
	fullscreen = !fullscreen;

	if(fullscreen)
	{
		window_width = glutGet(GLUT_WINDOW_WIDTH);
		window_height = glutGet(GLUT_WINDOW_HEIGHT);

		glutFullScreen();
		printf("Fullscreen MODE: ON\n");
	}
	else
	{
		glutReshapeWindow(window_width, window_height);
		glutPositionWindow(window_X_position, window_Y_position);
		printf("Fullscreen MODE: OFF\n");
	}
}

void keyboard(unsigned char key, int x, int y)
{

  int modif = 0;
  modif = glutGetModifiers();

  switch (toupper(key)) {
  case 'Q':
    _switch("ExitYN", 0);
    glutPostRedisplay();
    break;
  case 'W': togglewire();
    break;
  case 'H': print_help();
    break;
  case 'F': toggle_fullscreen();
    break;
  case '4':
    roty += 5;
    glutPostRedisplay();
    break;
  case '6':
    roty -= 5;
    glutPostRedisplay();
    break;
  case '8':
    rotx += 5;
    glutPostRedisplay();
    break;
  case '2':
    rotx -= 5;
    glutPostRedisplay();
    break;
  case '+':
    traz += .5;
    glutPostRedisplay();
    break;
  case '-':
    traz -= .5;
    glutPostRedisplay();
    break;
  case '7':
    rotx = 90;
    glutPostRedisplay();
    break;
  case '5':
    if (roty>0&&roty<=45) roty = 0;
    if (roty<0&&roty>=-45) roty = 0;
    if (roty>45&&roty<=135) roty = 90;
    if (roty<-45&&roty>-135) roty = -90;
    if (roty>135&&roty<=225) roty = 180;
    if (roty<-135&&roty>-225) roty = -180;
    if (roty>225&&roty<=315) roty = 270;
    if (roty<-225&&roty>-315) roty = -270;
    if (roty>315&&roty>0) roty = 0;
    if (roty<-315&&roty<0) roty = 0;
    rotx = 0.0;

    glutPostRedisplay();
    break;

  }

}



void specialKeyboard(int key, int x, int y)
{
  int modif = 0;
  modif = glutGetModifiers();

  switch (key) {
  case GLUT_KEY_F1:
    rotx = 0.0;
    roty = 0.0;
    trax = 0.0;
    tray = -2.0;
    traz = -2;
    glutPostRedisplay();
    break;
  case GLUT_KEY_F2:
    rotx = 0.0;
    roty = 0.0;
    trax = 0.0;
    tray = -1.2;
    traz = -3;
    glutPostRedisplay();
    break;
  case GLUT_KEY_F3:
    rotx = 0.0;
    roty = 0.0;
    trax = 0.0;
    tray = -0.2;
    traz = -3;
    glutPostRedisplay();
    break;
  case GLUT_KEY_F4:
    rotx = 0.0;
    roty = 0.0;
    trax = 0.0;
    tray = 1.0;
    traz = -3;
    glutPostRedisplay();
    break;
  case GLUT_KEY_F5:
    rotx = 0.0;
    roty = 0.0;
    trax = 0.0;
    tray = 2.2;
    traz = -3;
    glutPostRedisplay();
    break;
  case GLUT_KEY_LEFT:
    trax += .2;
    glutPostRedisplay();
    break;
  case GLUT_KEY_RIGHT:
    trax -= .2;
    glutPostRedisplay();
    break;
  case GLUT_KEY_UP:
    tray += .2;
    glutPostRedisplay();
    break;
  case GLUT_KEY_DOWN:
    tray -= .2;
    glutPostRedisplay();
    break;
  case GLUT_KEY_HOME:
    rotx = .0;
    roty = .0;
    trax = 0.0;
    tray = -0.2;
    traz = -9.5;
    glutPostRedisplay();
    break;
  case GLUT_KEY_F9:
    _render("rndr", 0);
    break;
  case GLUT_KEY_F10:
    _render("aqsis", 0);
    break;
  case GLUT_KEY_F11:
    _render("rndr", 1);
    break;
  case GLUT_KEY_F12:
    _render("aqsis", 1);
    break;
  }
}

void mouseMotion(int x, int y)
{
  OnOver(x,y);
}

void animTimer(int value){

  if (value){
    Animation();
    glutPostRedisplay();
  }
  glutTimerFunc(50, animTimer, anim);
}

void closesplash(int value){
    _switch("MHmain", 0);  
    glutPostRedisplay();    
}

void splash_timer(int value){
    _splashScreen("", 0);
    glutPostRedisplay();
    glutTimerFunc(1000, closesplash, 1);
    
}

void print_help()
{
	printf("'h'      	- help\n");
	printf("'f'      	- in/out fullscreen\n");
	printf("'w'      	- in/out wireframe\n");
	printf("'Shift LMB'	- pan\n");
	printf("'Ctrl LMB'	- Rotate\n");
	printf("'MMB'		- Rotate\n");
	printf("'Alt LMB'	- Zoom\n");
}


void init(int argc, char **argv)
{
	// Initialize window
	window_X_position = (glutGet(GLUT_SCREEN_WIDTH) - window_width)/2;
	window_Y_position = (glutGet(GLUT_SCREEN_HEIGHT) - window_height)/2;
	glutInitWindowSize(window_width, window_height);
	glutInitWindowPosition(window_X_position, window_Y_position);

	load_model(argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	glutCreateWindow(MHTITLE VERSION_STRING);
	SetUP_UI();

	_scanTargetDir();

	print_help();
    

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glMatrixMode(GL_PROJECTION);
  gluPerspective( /* field of view in degree */ 35.0,
      /* aspect ratio */ 1.0,
      /* Z near */ 0.1, /* Z far */ 100.0);
  glMatrixMode(GL_MODELVIEW);


  glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE,  white_light);
  glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
  glLightfv(GL_LIGHT0, GL_POSITION, left_light_position);

  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHTING);
  glEnable(GL_COLOR_MATERIAL);

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutMouseFunc(mouse);
	glutPassiveMotionFunc(mouseMotion);
	glutMotionFunc(motion);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialKeyboard);
    
    glutTimerFunc(50, animTimer, anim);
    glutTimerFunc(50, splash_timer, 1);
    
}


