/*
Copyright (c) 2001-2005 MakeHuman team www.makehuman.org
Author: Paolo Colombo

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifdef __APPLE__
#include <GLUT/glut.h>
#include <limits.h>
#include <sys/types.h>
#include <dirent.h>
#else
#include <GL/glut.h>
#include <GL/gl.h>
#endif

#ifdef _WIN32
#include <process.h>
#include <windows.h> //for win32 compile          
#endif

#include <stdio.h>
#include <string.h>  
#include <sys/types.h>
#include <sys/stat.h>

#include "io.h"
#include "glut_interface.h"
#include "make_human.h"
#include "target_list.h"
#include "ui.h"

void ScanDir(char *dirname)
{

  
#ifdef _WIN32
  BOOL            fFinished;
  HANDLE          hList;
  TCHAR           szDir[MAX_PATH];
  TCHAR           szSubDir[MAX_PATH];
  WIN32_FIND_DATA FileData;

  char buffer[MAX_PATH];

  counterTargets = 0;

  sprintf(szDir, "%s\\*", dirname);

  // Get the first file
  hList = FindFirstFile(szDir, &FileData);
  if (hList == INVALID_HANDLE_VALUE)
  {
    printf("No files found\n\n");
  }
  else
  {
    // Traverse through the directory structure
    fFinished = FALSE;
    while (!fFinished)
    {
      // Check the object is a directory or not
      if (FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      {
        if ((strcmp(FileData.cFileName, ".") != 0) && (strcmp(FileData.cFileName, "..") != 0))
        {
          // Get the full path for sub directory
          sprintf(szSubDir, "%s\\%s", dirname, FileData.cFileName);
          targetList[counterDir].dirName = malloc((strlen(FileData.cFileName)+1)*sizeof(char));
          strcpy(targetList[counterDir].dirName,FileData.cFileName);
          targetList[counterDir].id = counterDir;
          ScanDir(szSubDir);
          counterDir++;
        }

      }
      else{
          strcpy(buffer, FileData.cFileName);
          strtok(buffer, ".");
          
          do{
            strcpy(buffer, strtok(NULL, "."));
          }while((strtok(NULL, ".")!= NULL));
          
          if (strcmp(buffer, "bs") == 0 || strcmp(buffer, "target")==0){
                strcpy(targetList[counterDir].targetNames[counterTargets],FileData.cFileName);
                targetList[counterDir].numOfTargets = counterTargets+1;
                counterTargets++;
          }
      }
      if (!FindNextFile(hList, &FileData))
      {
        if (GetLastError() == ERROR_NO_MORE_FILES)
        {
          fFinished = TRUE;
        }
      }
    }
  }
  FindClose(hList);
  
#else // for non win32 plattforms

  bool file_exist;
  char full_path[MAX_PATH];
  char *dir_file;
  struct dirent *dir_entry;
  struct stat buf;
  DIR *dir;

  char buffer[MAX_PATH];
  
  counterTargets = 0;

  dir = opendir (dirname);

  if (!dir)
  {
    printf("No files found\n\n");
    return;
  }

  do
  {
    dir_entry = readdir (dir);


    if (dir_entry)
    {
      dir_file = dir_entry->d_name;

      sprintf (full_path, "%s/%s", dirname, dir_file);

      file_exist = !(stat (full_path, &buf));

      if (file_exist && (dir_file[0] !=  '.'))
      {
        if (S_ISDIR  (buf.st_mode))
        {
          targetList[counterDir].dirName = malloc((strlen(dir_file)+1) * sizeof(char));
          strcpy(targetList[counterDir].dirName, dir_file);
          targetList[counterDir].id = counterDir;
          ScanDir(full_path);
          counterDir++;

        }
        else if (S_ISREG (buf.st_mode))
        {
          /*printf ("file: %s\n", full_path);*/
          /* control ".target" extension!*/

          //targetList[counterDir].targetNames[counterTargets] = malloc ((strlen (dir_file)+1) * sizeof(char));
          // why no malloc here? -> perhaps I don't yet understand the data structure
          strcpy(buffer, dir_file);
          strtok(buffer, ".");
          
          do{
            strcpy(buffer, strtok(NULL, "."));
          }while((strtok(NULL, ".")!= NULL));
          
          if (strcmp(buffer, "bs") == 0 || strcmp(buffer, "target")==0){
            strcpy(targetList[counterDir].targetNames[counterTargets], dir_file);
            targetList[counterDir].numOfTargets = counterTargets+1;
            counterTargets++;
          }

        }
      }
    }
  }
  while (dir_entry);

#endif
}

char *getHomeDir ()
{
#ifndef _WIN32
  char *pw_dir;
  uid_t uid;
  struct passwd *pass;

  uid = getuid();
  pass = getpwuid (uid);
  pw_dir = pass->pw_dir;
  
  return strdup (pw_dir);

#else // return empty string for win32
  return strdup (".");
#endif
}

char *getAppHomeDir ()
{
#ifndef _WIN32
  char *homedir;
  char *app_home_dir;
  int size;

  homedir = getHomeDir ();
  size = strlen (homedir) + strlen (APP_HOME) + 2;
  app_home_dir = malloc (size);

  snprintf (app_home_dir, size, "%s/%s", homedir, APP_HOME);

  return app_home_dir;
#else
  return strdup (".");
#endif
}


int initAppHome ()
{
  int retval;
  int fd;
  char path[MAX_PATH];

#ifndef _WIN32  
  retval = mkdir(getAppHomeDir (),
                 S_IRUSR | S_IWUSR | S_IXUSR |
                 S_IRGRP | S_IXGRP |
                 S_IROTH | S_IXOTH);

  sprintf (path, "%s/bodysettings", getAppHomeDir ());
  retval = mkdir(path,
                 S_IRUSR | S_IWUSR | S_IXUSR |
                 S_IRGRP | S_IXGRP |
                 S_IROTH | S_IXOTH);

  sprintf (path, "%s/exported", getAppHomeDir ());
  retval = mkdir(path,
                 S_IRUSR | S_IWUSR | S_IXUSR |
                 S_IRGRP | S_IXGRP |
                 S_IROTH | S_IXOTH);

  sprintf (path, "%s/rib", getAppHomeDir ());
  retval = mkdir(path,
                 S_IRUSR | S_IWUSR | S_IXUSR |
                 S_IRGRP | S_IXGRP |
                 S_IROTH | S_IXOTH);

  sprintf (path, "%s/rib/images", getAppHomeDir ());
  retval = mkdir(path,
                 S_IRUSR | S_IWUSR | S_IXUSR |
                 S_IRGRP | S_IXGRP |
                 S_IROTH | S_IXOTH);

  sprintf (path, "%s/rib/ribobjs", getAppHomeDir ());
  retval = mkdir(path,
                 S_IRUSR | S_IWUSR | S_IXUSR |
                 S_IRGRP | S_IXGRP |
                 S_IROTH | S_IXOTH);

#endif

#ifdef _WIN32
//  retval = mkdir(getAppHomeDir ());


#endif
  return 0;
}


void _scanTargetDir()
{

  ScanDir(DATA_DIR);

}


void Animation(){

  /*
  	extern int anim;
  	static int loaded = 0;
  	static int applyed = 0;
  	static int cycle = 0;
  	static int firsttime = 0;
  	static char *animItems[] = {
  	"targets/cinematics/01.target",
  	"targets/cinematics/02.target",
  	"targets/cinematics/03.target",
  	};

  	if (!firsttime){
  		Load_Target_Anim(animItems[cycle]);
  		Do_Morph_Anim(animItems[cycle], 1.0);
  		firsttime =1;
  	}
  	else{
  		Do_Morph_Anim(animItems[cycle-1], -1.0);
  		Load_Target_Anim(animItems[cycle]);
  		Do_Morph_Anim(animItems[cycle], 1.0);
  	}

  	cycle += 1;
  	if (cycle > 2) cycle = 0;

  	if (!anim){
  		Do_Morph_Anim(animItems[cycle], -1.0);
  		cycle = 0;
  		firsttime = 0;
  		}
  	glutPostRedisplay();

  	if (!loaded){
  		Load_Target_Anim(animItems[cycle]);
  		loaded = 1;
  		}
  	if(!applyed) {
  		Do_Morph_Anim(animItems[cycle], 1.0);
  		applyed =1;
  	}else{
  		Do_Morph_Anim(animItems[cycle-1], -1.0);
  		loaded = 0;
  		applyed = 0;
  		}
  	cycle += 1;
  	if (cycle > 2) cycle = 0;

  	glutPostRedisplay();
  */
  extern int anim;
  static int firsttime = 0;
  static int updwn = 1;
  static float val = 0.0;
  //Do_Morph(target, alue);
  if (updwn){
    val += 0.05;
    if (val>1) updwn = 0;
  }
  else{
    val-=0.05;
    if (val<0){
      val = 0.0;
      updwn = 1;
    }
  }
  if (!firsttime){
#ifdef HAVE_CONFIG_H
    Load_Target_Anim(PACKAGE_DATA_DIR"/targets/cinematics/cinematic.target");
#else
    Load_Target_Anim("targets/cinematics/cinematic.target");
#endif
    firsttime = 1;
  }
#ifdef HAVE_CONFIG_H
  Do_Morph_Anim(PACKAGE_DATA_DIR"/targets/cinematic/cimematic.target", val);
#else
  Do_Morph_Anim("targets/cinematic/cimematic.target", val);
#endif

  if (!anim){
#ifdef HAVE_CONFIG_H
    Do_Morph_Anim(PACKAGE_DATA_DIR"/targets/cinematic/cimematic.target", 0.0);
#else
    Do_Morph_Anim("targets/cinematic/cimematic.target", 0.0);
#endif
    //cycle = 0;
    firsttime = 0;
    updwn = 1;
    val = 0.0;
  }

}

int LoadBodysetting(char *name){

  FILE *file;
  char buf[80];
  const char *util;
  char targetname[80];
  float value;
  char token[] = ",";
  int counter;

  setToZero(TargetUsed);
  printf("Loading Bodysetting\n");

  file = fopen(name, "r");

  if(!file)
  {
    printf("Errore nella lettura del file\n");
    return 1;
  }
  counter =0;
  while( fgets( buf, 80, file ))
  {
    util = strtok(buf, token);
    strcpy(targetname, util);
    util = strtok (NULL, token);
    value = atof(util);

#ifdef HAVE_CONFIG_H
    char tmp[MAX_PATH];
    strcpy (tmp, PACKAGE_DATA_DIR"/");
    strcat (tmp, targetname);
    if(Load_Target(tmp)==0){
#else
    if(Load_Target(targetname)==0){
#endif
      //printf("il file esiste!!!!!\n");
      Do_Morph(targetname, value);
      display();
    }
    //printf("Eccoci\n");

  }

  fclose(file);
  calculate_vertex_normal();

	// Acknowledge target update
	//update_mesh();

  _buildSliders("", 1);

  return 0;
}


void SaveBodysetting(char *name){

  FILE *file;
  int n = 0;
  char filename[MAX_PATH];
  int file_not_exist;
  struct stat buf;
  char *app_homedir = getAppHomeDir ();

  TargetUsedStruct *point;
  point = TargetUsed;
  
 // test which filename is still free
  do
  {
    n++;
    sprintf (filename, "%s/bodysettings/bodysettings%d.bs", app_homedir, n);
    file_not_exist = !(stat (filename, &buf));

  } while (file_not_exist);
  
  file = fopen(filename, "w");
  if(!file)
  {
    printf("Error while creating %s to save Bodysettings!\n",filename);
    exit(1);
  }
  while (point != NULL)
  {
    if(point->Value != 0){
      fprintf(file, "%s,%f\n", point->TargetName, point->Value);
      //point = point->next;
    }
    point = point->next;
  }
  fclose(file);
  printf ("Bodysetting File: %s saved!\n", filename);

  free (app_homedir);
}

void writeRib(FILE *ribfileout, char *imgfile, int xres, int yres){

  extern GLfloat focal, rotx, trax, tray, traz;

  ////////////////////////////HEADER

  fprintf(ribfileout, "Option \"statistics\" \"endofframe\" [1]\n");
  fprintf(ribfileout, "Projection \"perspective\" \"fov\" [%f]\n", focal);
  fprintf(ribfileout, "Format %d %d 1\n", xres, yres);
  fprintf(ribfileout, "Clipping 0.1 100.0\n");
  fprintf(ribfileout, "PixelSamples 2 2\n");
  fprintf(ribfileout, "Sides 2\n");
  fprintf(ribfileout, "Display \"00001.tif\" \"framebuffer\" \"rgb\"\n");
  if(imgfile != "")	fprintf(ribfileout, "Display \"+%s\" \"file\" \"rgba\"\n",imgfile);
  fprintf(ribfileout, "\t\tTranslate %f %f %f\n", trax, tray, traz*-1);
  fprintf(ribfileout, "\t\tRotate %f 1 0 0\n", rotx*-1);
  fprintf(ribfileout, "WorldBegin\n");
  fprintf(ribfileout, "\tLightSource \"ambientlight\" 998 \"intensity\" [0.15] \"color lightcolor\" [1.0 1.0 1.0] \n\n");
}

void writeLamps(FILE *ribfileout){

  fprintf(ribfileout, "LightSource \"pointlight\" 1 \"from\" [1.34805500507 2.71158885956 -3.07836914063] \"intensity\" 12  \"lightcolor\" [1.0 1.0 1.0]  \n\n");
  fprintf(ribfileout, " LightSource \"pointlight\" 2 \"from\" [-1.19248187542 -1.36335754395 -2.81925439835] \"intensity\" 10  \"lightcolor\" [1.0 1.0 1.0]  \n\n");
}

int saveOBJ (/*char *filename*/)
{
  FILE *file;
  int i, j;
  int n = 0;
  char filename[MAX_PATH];
  int file_not_exist;
  struct stat buf;
  char *app_homedir = getAppHomeDir ();

  extern GLint vertex, faces, *Tri_quad, **face_index;
  extern GLdouble **vertex_coor;

  // test which filename is still free
  do
  {
    n++;
    sprintf (filename, "%s/exported/outfile%d.obj", app_homedir, n);
    file_not_exist = !(stat (filename, &buf));

  } while (file_not_exist);

  file = fopen(filename, "w");
  if (!file)
  {
    printf("Error while creating %s to save OBJ!\n", filename);
    return -1;
  }

  // save header
  fprintf (file, "# MakeHuman OBJ File\n");
  fprintf (file, "# http://www.dedalo-3d.com/\n"); // website info
  fprintf (file, "o human_mesh\n"); // name of mesh

  // save vertices
  for(i = 0; i < vertex; i++)
  {
    fprintf(file,"v %f %f %f\n" ,vertex_coor[i][0], vertex_coor[i][1] ,vertex_coor[i][2]);
  }

  // save faces list
  for(i = 0; i < faces; i++)
  {
    fprintf (file, "f");
    for(j = 0; j < Tri_quad[i]; j++)
    {
      fprintf (file, " %d//", face_index[i][j]+1);
    }
    fprintf (file, "\n");
  }

  fclose(file);

  printf ("OBJ File: %s saved!\n", filename);
  free (app_homedir);

  return 0;
}

void subdivmesh(char *fileObj){

  FILE *file;
  int i, j;
  extern GLint vertex, faces, *Tri_quad, **face_index;
  extern GLdouble **vertex_coor;
  char *app_homedir = getAppHomeDir ();
  char path[MAX_PATH];

  file = fopen(fileObj, "w");

  if (!file){
    printf("Errore nella creazione del file .RIB dell'oggetto suddiviso\n");
    exit(1);
  }

  fprintf(file,"SubdivisionMesh \"catmull-clark\" [");
  for(i=0; i<faces; i++){
    ///	objFile.write('%s '%(num))
    fprintf(file,"%d ", Tri_quad[i]);
  }
  fprintf(file,"]\n[");
  for(i=0; i<faces; i++){
    for(j=0; j<Tri_quad[i]; j++){
      fprintf(file,"%d ", face_index[i][j]);
    }
  }
  fprintf(file,"]\n[\"interpolateboundary\"] [0 0] [] []\n\"P\" [");

  for(i=0; i<vertex; i++){
    fprintf(file,"%f %f %f " ,vertex_coor[i][0], vertex_coor[i][1] ,vertex_coor[i][2]*-1);
  }

  fprintf(file,"]");

  fclose(file);
  free (app_homedir);
}

void writeObjs(FILE *ribfileout, char *ribObjDir){

  extern GLfloat roty;

  fprintf(ribfileout,"\tAttributeBegin\n");
  fprintf(ribfileout, "\t\tRotate 90 1 0 0\n");
  fprintf(ribfileout, "\t\tRotate %f 0 0 1\n",roty);
  fprintf(ribfileout, "\t\tScale 0.3 0.3 0.3 \n");
  fprintf(ribfileout,"\tColor [0.930000007153 0.829999983311 0.810000002384]\n");
  fprintf(ribfileout,"\tOpacity [1.0 1.0 1.0]\n");
  subdivmesh(ribObjDir);
  fprintf(ribfileout,"\t\tSurface \"plastic\"\n");
  fprintf(ribfileout,"\t\tReadArchive \"%s\"\n" , ribObjDir);
  fprintf(ribfileout,"\tAttributeEnd\n");
}

void MakeRib(int mode, char *fName, char *imgName, int xres, int yres, char *engine){

  FILE *ribfileout;

  char ribObjDir[MAX_PATH];
  char ribFile[MAX_PATH];
  char ribImage[MAX_PATH];
  char currentDir[MAX_PATH];
  char renderer[20];
  char *app_homedir = getAppHomeDir ();
  //char path[MAX_PATH];

#if HAVE_CONFIG_H
  strcpy (currentDir, app_homedir);
#else
  getcwd(currentDir, MAX_PATH);
#endif

  strcpy(ribImage, currentDir);
  strcat(ribImage, "/rib/images/");
  strcat(ribImage, imgName);

  strcpy(ribFile, currentDir);
  strcat(ribFile, "/rib/");
  strcat(ribFile, fName);

  strcpy(ribObjDir, currentDir);
  strcat(ribObjDir, "/rib/ribobjs/");
  strcat(ribObjDir, "objs_");
  strcat(ribObjDir, fName);

  //printf ("path: %s\n", ribFile);
  ribfileout = fopen(ribFile, "w");
  if (!ribfileout){
    printf("Errore nella creazione del file .RIB\n");
    exit(1);
  }

  //printf ("image: %s\n", ribImage);
  writeRib(ribfileout, ribImage, xres, yres);

  writeLamps(ribfileout);
  //printf ("objs: %s\n", ribObjDir);
  writeObjs(ribfileout, ribObjDir);
  fprintf(ribfileout,"WorldEnd\n");
  fclose(ribfileout);

  strcpy(renderer, engine);

#ifdef _WIN32
  commandExec(strcat(renderer, ".exe"), renderer, ribFile);
#else
  commandExec(renderer, renderer, ribFile);
#endif

  free (app_homedir);
}

void commandExec(char *command, char *par1, char *param)
{
#ifndef _WIN32
  // something is not working here!!

  /*int ret_val;
  pid_t exec_pid;

  exec_pid = fork ();
  if (exec_pid == 0)
  {
    ret_val = execlp (command, par1, param, NULL);
    if (ret_val == -1)
    {
      printf ("couldn't start renderer\n");
      return 1;
    }
  }*/

  char popen_command[MAX_PATH];
  FILE *wait;

  sprintf (popen_command, "%s %s", command, param);
  wait = popen (popen_command, "r");
  pclose (wait);

  //free (popen_command);

#else // _WIN32
  spawnlp(P_NOWAIT,command,par1,param,NULL); // BY AV
#endif
}

